@@ -2,13 +2,14 @@ package HTML::FormFu::Element::Button;
 
 use strict;
 use base 'HTML::FormFu::Element::_Input';
-use Class::C3;
+use MRO::Compat;
+use mro 'c3';
 
 sub new {
     my $self = shift->next::method(@_);
 
-    $self->field_type    ( 'button' );
-    $self->retain_default( 1 );
+    $self->field_type('button');
+    $self->force_default(1);
 
     return $self;
 }
@@ -50,3 +51,5 @@ Carl Franks, C<cfranks@cpan.org>
 
 This library is free software, you can redistribute it and/or modify it under
 the same terms as Perl itself.
+
+=cut
