@@ -2,13 +2,14 @@ package HTML::FormFu::Element::Hidden;
 
 use strict;
 use base 'HTML::FormFu::Element::_Input';
-use Class::C3;
+use MRO::Compat;
+use mro 'c3';
 
 sub new {
     my $self = shift->next::method(@_);
 
-    $self->field_type( 'hidden' );
-    $self->filename  ( 'input_tag' );
+    $self->field_type('hidden');
+    $self->filename('input_tag');
 
     return $self;
 }
@@ -18,9 +19,10 @@ sub string {
 
     $args ||= {};
 
-    my $render = exists $args->{render_data} ? $args->{render_data}
-               :                               $self->render_data
-               ;
+    my $render
+        = exists $args->{render_data}
+        ? $args->{render_data}
+        : $self->render_data;
 
     # input template
 
@@ -64,3 +66,5 @@ Carl Franks, C<cfranks@cpan.org>
 
 This library is free software, you can redistribute it and/or modify it under
 the same terms as Perl itself.
+
+=cut
