@@ -2,11 +2,12 @@ package HTML::FormFu::Element::Password;
 
 use strict;
 use base 'HTML::FormFu::Element::_Input';
-use Class::C3;
+use MRO::Compat;
+use mro 'c3';
 
 use HTML::FormFu::Constants qw( $EMPTY_STR );
 
-__PACKAGE__->mk_item_accessors( qw( render_value ) );
+__PACKAGE__->mk_item_accessors(qw( render_value ));
 
 sub new {
     my $self = shift->next::method(@_);
@@ -23,9 +24,10 @@ sub process_value {
     my $new;
 
     if ( $submitted && $self->render_value ) {
-        $new = defined $value ? $value
-             :                  $EMPTY_STR
-             ;
+        $new
+            = defined $value
+            ? $value
+            : $EMPTY_STR;
 
         if ( $self->retain_default && $new eq $EMPTY_STR ) {
             $new = $self->value;
@@ -61,6 +63,15 @@ Password form field.
 
 =head1 METHODS
 
+=head2 render_value
+
+Normally, when a form is redisplayed because of errors, password fields
+lose their values, requiring the user to retype it.
+
+If C<render_value> is true, password fields won't lose their value.
+
+Default value: false
+
 =head1 SEE ALSO
 
 Is a sub-class of, and inherits methods from 
@@ -78,3 +89,5 @@ Carl Franks, C<cfranks@cpan.org>
 
 This library is free software, you can redistribute it and/or modify it under
 the same terms as Perl itself.
+
+=cut
