@@ -2,11 +2,12 @@ package HTML::FormFu::Element::Radio;
 
 use strict;
 use base 'HTML::FormFu::Element::Checkbox';
-use Class::C3;
+use MRO::Compat;
+use mro 'c3';
 
 use HTML::FormFu::Constants qw( $EMPTY_STR );
 
-__PACKAGE__->mk_output_accessors( qw( default ) );
+__PACKAGE__->mk_output_accessors(qw( default ));
 
 sub new {
     my $self = shift->next::method(@_);
@@ -56,3 +57,5 @@ Carl Franks, C<cfranks@cpan.org>
 
 This library is free software, you can redistribute it and/or modify it under
 the same terms as Perl itself.
+
+=cut
