@@ -2,7 +2,8 @@ package HTML::FormFu::Element::Src;
 
 use strict;
 use base 'HTML::FormFu::Element::Block';
-use Class::C3;
+use MRO::Compat;
+use mro 'c3';
 
 sub new {
     my $self = shift->next::method(@_);
@@ -41,3 +42,5 @@ Carl Franks, C<cfranks@cpan.org>
 
 This library is free software, you can redistribute it and/or modify it under
 the same terms as Perl itself.
+
+=cut
