@@ -2,7 +2,8 @@ package HTML::FormFu::Element::Submit;
 
 use strict;
 use base 'HTML::FormFu::Element::Button';
-use Class::C3;
+use MRO::Compat;
+use mro 'c3';
 
 sub new {
     my $self = shift->next::method(@_);
@@ -30,10 +31,6 @@ Submit button form field.
 
 =head1 METHODS
 
-=head1 non_block
-
-See L<HTML::FormFu::Elements::_Field/non_block> for details.
-
 =head1 SEE ALSO
 
 Is a sub-class of, and inherits methods from 
@@ -52,3 +49,5 @@ Carl Franks, C<cfranks@cpan.org>
 
 This library is free software, you can redistribute it and/or modify it under
 the same terms as Perl itself.
+
+=cut
