@@ -14,13 +14,20 @@ sub nested_names {
         # ignore immediate parent
         my $parent = $self->parent;
 
-        while ( defined ( $parent = $parent->parent ) ) {
+        while ( defined( $parent = $parent->parent ) ) {
 
             if ( $parent->can('is_field') && $parent->is_field ) {
+                # handling Field
                 push @names, $parent->name
                     if defined $parent->name;
             }
+            elsif ( $parent->can('is_repeatable') && $parent->is_repeatable ) {
+                # handling Repeatable
+                # ignore Repeatables nested_name attribute as it is provided
+                # by the childrens Block elements
+            }
             else {
+                # handling 'not Field' and 'not Repeatable'
                 push @names, $parent->nested_name
                     if defined $parent->nested_name;
             }
@@ -42,7 +49,7 @@ sub nested_base {
     # ignore immediate parent
     my $parent = $self->parent;
 
-    while ( defined ( $parent = $parent->parent ) ) {
+    while ( defined( $parent = $parent->parent ) ) {
 
         return $parent->nested_name if defined $parent->nested_name;
     }
@@ -64,3 +71,5 @@ Carl Franks, C<cfranks@cpan.org>
 
 This library is free software, you can redistribute it and/or modify it under
 the same terms as Perl itself.
+
+=cut
