@@ -1,7 +1,8 @@
 package HTML::FormFu::Element::_MultiSelect;
 
 use strict;
-use base 'HTML::FormFu::Element::_MultiElement', 'HTML::FormFu::Element::Select';
+use base 'HTML::FormFu::Element::_MultiElement',
+    'HTML::FormFu::Element::Select';
 
 1;
 
@@ -19,3 +20,5 @@ Carl Franks, C<cfranks@cpan.org>
 
 This library is free software, you can redistribute it and/or modify it under
 the same terms as Perl itself.
+
+=cut
