@@ -2,11 +2,12 @@ package HTML::FormFu::Filter::HTMLScrubber;
 
 use strict;
 use base 'HTML::FormFu::Filter';
-use Class::C3;
+use MRO::Compat;
+use mro 'c3';
 
-use Storable qw( dclone );
+use Clone ();
 
-__PACKAGE__->mk_accessors( qw( allow ) );
+__PACKAGE__->mk_accessors(qw( allow comment default rules script ));
 
 use HTML::Scrubber;
 
@@ -15,9 +16,13 @@ sub filter {
 
     return if !defined $value;
 
-    my $allowed = $self->allow || [];
+    my %params = ( allow => 0 );
+    foreach (qw(allow comment default rules script)) {
+        my $val = $self->$_;
+        $params{$_} = $val if ( defined($val) );
+    }
 
-    my $scrubber = HTML::Scrubber->new( allow => $allowed );
+    my $scrubber = HTML::Scrubber->new(%params);
 
     return $scrubber->scrub($value);
 }
@@ -27,7 +32,7 @@ sub clone {
 
     my $clone = $self->next::method(@_);
 
-    $clone->allow( dclone $self->allow )
+    $clone->allow( Clone::clone $self->allow )
         if ref $self->allow;
 
     return $clone;
@@ -45,10 +50,21 @@ HTML::FormFu::Filter::HTMLScrubber - filter removing HTML markup
 
 Remove HTML markup using L<HTML::Scrubber>.
 
+All the functionality of L<HTML::Scrubber> can be accessed using
+this module, other than the C<process> directive (which has a name
+clash with the L<HTML::FormFu::Filter> framework).
+
+For details of the filtering functionality see
+L<HTML::Scrubber/allow>, L<HTML::Scrubber/comment>,
+L<HTML::Scrubber/default>, L<HTML::Scrubber/rules> and
+L<HTML::Scrubber/script>
+
 =head1 AUTHOR
 
 Carl Franks, C<cfranks@cpan.org>
 
+Extended by Nigel Metheringham, C<nigelm@cpan.org>
+
 Based on the original source code of L<HTML::Widget::Filter::HTMLStrip>, by 
 Lyo Kato, C<lyo.kato@gmail.com>
 
