@@ -1,22 +1,27 @@
 package HTML::FormFu::Model;
 
 use strict;
-use Class::C3;
+use MRO::Compat;
+use mro 'c3';
 
 use HTML::FormFu::Attribute qw( mk_accessors mk_item_accessors );
 use HTML::FormFu::ObjectUtil qw( form parent );
-use Scalar::Util qw( refaddr );
+use Scalar::Util qw( refaddr reftype );
 use Carp qw( croak );
 
-__PACKAGE__->mk_item_accessors( qw( type ) );
+__PACKAGE__->mk_item_accessors(qw( type ));
 
 sub new {
     my $class = shift;
-
     my %attrs;
-    eval { %attrs = %{ $_[0] } if @_ };
-    croak "attributes argument must be a hashref" if $@;
-
+    
+    if (@_) {
+        croak "attributes argument must be a hashref"
+            if reftype( $_[0] ) ne 'HASH';
+        
+        %attrs = %{ $_[0] };
+    }
+    
     my $self = bless {}, $class;
 
     for my $arg (qw( parent type )) {
@@ -74,3 +79,5 @@ Carl Franks, C<cfranks@cpan.org>
 
 This library is free software, you can redistribute it and/or modify it under
 the same terms as Perl itself.
+
+=cut
