@@ -2,7 +2,8 @@ package HTML::FormFu::Plugin::StashValid;
 
 use strict;
 use base 'HTML::FormFu::Plugin';
-use Class::C3;
+use MRO::Compat;
+use mro 'c3';
 
 sub post_process {
     my ($self) = @_;
@@ -65,3 +66,5 @@ Carl Franks C<cfranks@cpan.org>
 
 This library is free software, you can redistribute it and/or modify it under
 the same terms as Perl itself.
+
+=cut
